/***
* 	bpl_openwindslop.c -- module for bplib.lib
*   1993 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
  18 Oct 93 | created from bpl_newwindow.c to support leaving space for gadgets
*/

#include <exec/types.h>
#include <intuition/intuition.h>
#include <proto/intuition.h>
#include <string.h>
#include "bars.h"
#include "barsmacros.h"
#include "proto/bpl_openwindslop_protos.h"

extern struct IntuitionBase *IntuitionBase;
extern struct Functions * far functions;

extern char *MakeEditWindowTitle(struct Tool *, const char *, long );

/*==========================================================================
* OpenEmbossedWindowSlop
*
*  INPUT: pointer to NewWindow and tool, window flags, pointer to window title,
*         pixels of slop to save
* RETURN: pointer to NewWindow and pointer to Window in tool->window if
*         opened, else NULL
*
* EFFECT: twiddles with NewWindow and opens embossed window.  if flags==0,
*         defaults are used.  slop is subtracted from the maximum width of
*         the title to allow room for adding gadgets to the title bar area.
*         If the resulting title bar width is smaller than the nw.Width, the
*         window gets just the title passed.
*=========================================================================*/

struct NewWindow *OpenEmbossedWindowSlop(new, tool, flags, title, slop)

struct NewWindow *new;
struct Tool *tool;
ULONG flags;
const char *title;
SHORT slop;

{
	struct NewWindow *nw;
	char *wtitle;

	nw = (struct NewWindow *)functions->DupeNewWindow(new);
	if (nw) {
		if (tool->touched & TOUCH_EDIT) {
			nw->LeftEdge = tool->left;
			nw->TopEdge = tool->top;
			nw->Width = tool->width;
			nw->Height = tool->height;
		}
		else { /* put window in the vicinity of the tool icon */
			nw->TopEdge = tool->y+23; /* at the bottom of the icon */
			if (nw->TopEdge + nw->Height > functions->screen->Height) {
				nw->TopEdge = functions->screen->Height - nw->Height;
			}
			nw->LeftEdge = tool->x-(nw->Width >> 1)+12; /* centered with icon */
			if (nw->LeftEdge + nw->Width > functions->screen->Width) {
				nw->LeftEdge = functions->screen->Width - nw->Width;
			}
			else if (nw->LeftEdge < 0) nw->LeftEdge=0;
		}
		nw->Screen = functions->screen;
		nw->Title = 0;
		nw->Flags |= BORDERLESS;
		nw->Flags &= ~0xF;
		nw->DetailPen = nw->BlockPen = 0;
		tool->window = (struct Window *)functions->FlashyOpenWindow(nw);
		if (tool->window) {
			if (slop < 0) slop *= -1; /* allow for stupidity */
			/* subtracting 55 allows for close & front gadget; drag text offset */
			slop = (nw->Width-55-slop) >> 3; /*! 8 pixel font alert !*/
			wtitle = MakeEditWindowTitle(tool,title,slop);
			functions->EmbossWindowOn(tool->window, flags ?
			                          flags : WINDOWCLOSE|WINDOWDEPTH|WINDOWDRAG,
											  wtitle,(short)-1,(short)-1,0,0);
		}
		else {
			functions->DeleteNewWindow(nw);
			nw=NULL;
		}
	}
	return(nw);
}

