/***
* 	bplib_bozo.c -- easy requester module for bplib.lib
*   1993, 1994  Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
  22 Jun 93 | created
  23 Jun 93 | improved
  29 Jun 93 | updated from m:lorenz.c
  30 Jun 93 | updated from m:lorenz.c; split into three functions
   8 Jul 93 | final update from m:lorenz.c
  11 Feb 94 | less space between body and gadgets; char * const now
*/

#include <exec/types.h>
#include <exec/memory.h>
#include <graphics/gfx.h>
#include <graphics/gfxmacros.h>
#include <intuition/intuition.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <string.h>
#include "bars.h"
#include "proto/bpl_bozo_protos.h"

extern struct IntuitionBase *IntuitionBase;
extern struct GfxBase *GfxBase;
extern struct Functions * far functions;

/* this variable is used if the user does not pass a NewWindow * */
/* this also means that only one BozoRequest can be opened with out a
 * caller-provided NewWindow
 */
static struct NewWindow *BR_nw = NULL;

/*** Window stuff ***/
/* add slop above and below bodytext */
#define BR_HEADROOM 10
/* add slop before and after bodytext horizontally */
#define BR_LEGROOM 15
/* pixels between lines of bodytext */
#define BR_LINESPACE 4
/* height of the window's title bar */
#define TITLEBAR_HEIGHT 11
/* default text color */
#define BR_TEXTCOLOR (SHORT)0
/* default bpen & window fill color */
#define BR_FILLCOLOR (SHORT)2

/*** Gadget stuff ***/
/* space above and below gadget text (IText.TopEdge) */
#define BR_GHEADROOM 2
/* space before and after gadget text (IText.LeftEdge) */
#define BR_GLEGROOM 4
/* gadget text color */
#define BR_GTEXTCOLOR (SHORT)4
/* gadget offset from window Edge, and minimum distance from each other */
#define BR_GXOFFSET (SHORT)15
/* gadget offset from bottom of window and body text */
#define BR_GYOFFSET (SHORT)8


/* GadgetIDs and return values (also available in bplib.h) */
#define BR_YES 1
#define BR_NO 2
#define BR_CLOSEWINDOW 3
#define BR_ERRORRETURN -1


/*==========================================================================
* OpenBozoRequest
*
*  INPUT: pointer to NewWindow, pointers to title and body text, window flags,
*         apen and bpen colors, center flag, pointer to yes and no gadget text.
*
* RETURN: pointer to Window
*
* EFFECT: opens the requester window, allocates and defines gadgets, writes
*         the text.
*
*			 See BozoRequest() for full details.
*
*=========================================================================*/

struct Window *OpenBozoRequest(nw, title, body, flags, apen, bpen, center, 
										 yestext, notext)
struct NewWindow *nw;
char * const title;
char * const body;
ULONG flags;
short apen;
short bpen;
int center;
char * const yestext;
char * const notext;

{
	struct Window *w;
	struct Gadget *glist;
	char *c;
	char *p;
	UBYTE nl; /* count newlines */
	UBYTE l;  /* length of the longest line */
	USHORT gcount = 0;

	if (!nw) {
		if (BR_nw) return(NULL); /* accomodate only one request at a time */
		if (!(BR_nw = (struct NewWindow *)
				functions->myalloc(sizeof(struct NewWindow),MEMF_CLEAR))) {
			return(NULL);
		}
		else nw = BR_nw;
	}

	nw->Title = 0;
	nw->Flags |= BORDERLESS|ACTIVATE;
	nw->Flags &= ~0xF;
	nw->DetailPen = nw->BlockPen = 0;
	nw->Screen = functions->screen;
	nw->Type = CUSTOMSCREEN;

	/** now, figure out how to fit the bozo's text in this puppy **/
	for (nl=l=0,p=c=body ; *c ; c++) { /* scan the body text */
		if (*c == '\n') { /* count newlines */
			nl++;
			if (c - p > l) l=c-p; /* remember the length of the longest line */
			p = c+1; /* point at char after this newline */
		}
	}
	if (nl) { /* multiple lines; get length of last one */
		if (c - p > l) l=c-p;
	}
	if (!l) l = strlen(body); /* one line of text */

	/* calculate the window dimensions sans gadgetry */
	nw->Height = (nl * BR_LINESPACE) + (++nl << 2) + (BR_HEADROOM << 1);
	if (nw->Height > functions->screen->Height) 
		nw->Height = functions->screen->Height;
	if ((flags & WINDOWDRAG) || title) nw->Height += TITLEBAR_HEIGHT;
	if (nw->Height > functions->screen->Height) 
		nw->Height = functions->screen->Height;

	if (title && (strlen(title) > l))
		nw->Width = strlen(title) << 3; /* title string longer than longest text */
	else nw->Width = (l << 3) + (BR_LEGROOM << 1); /* long text line is longest */
	if (nw->Width > functions->screen->Width) 
		nw->Width = functions->screen->Width;

	/* count, allocate, and define gadgetry */
	if (yestext) { /* if there is only one gadget, it shall be BR_YES */
		gcount=1;
		if (notext) gcount++;
		if (glist = (struct Gadget *)
	             functions->myalloc(sizeof(struct Gadget)*gcount,MEMF_CLEAR)) {
			memset((char *)glist,0,sizeof(struct Gadget)); /* clear first; memcpy() does next */
			if (glist->GadgetText = (struct IntuiText *)
				   functions->myalloc(sizeof(struct IntuiText)*gcount,MEMF_CLEAR)) {
				memset((char *)(glist->GadgetText),0,sizeof(struct IntuiText));
				glist->Height = 8 + (BR_GHEADROOM * 2);
				if (notext) glist->Width = (strlen(yestext) > strlen(notext)) ? 
			                   strlen(yestext) * 8 + (BR_GLEGROOM * 2) :
								    strlen(notext) * 8 + (BR_GLEGROOM * 2) ;
				else glist->Width = strlen(yestext) * 8 + (BR_GLEGROOM * 2);
				/* adjust window size to make room for gadgets */
				nw->Height += 8 + glist->Height + BR_GYOFFSET;
				if (nw->Height > functions->screen->Height) 
					nw->Height = functions->screen->Height;
				if ((glist->Width * gcount)+BR_GXOFFSET*3 > nw->Width) 
						nw->Width = glist->Width * 2 + BR_GXOFFSET * 3;
				if (nw->Width > functions->screen->Width) 
					nw->Width = functions->screen->Width;
				glist->TopEdge = nw->Height - glist->Height - BR_GYOFFSET;
				glist->LeftEdge = (gcount>1) ? BR_GXOFFSET : 
				                   nw->Width - glist->Width - BR_GXOFFSET;
				glist->Activation = GADGIMMEDIATE | RELVERIFY;
				glist->GadgetType = BOOLGADGET;
				glist->GadgetID = BR_YES;
				/* set up IText */
				glist->GadgetText->IText = yestext;
				glist->GadgetText->FrontPen = BR_GTEXTCOLOR;
				glist->GadgetText->DrawMode = JAM1;
				glist->GadgetText->LeftEdge = (glist->Width-strlen(yestext)*8) / 2;
				glist->GadgetText->TopEdge = BR_GHEADROOM;
				if (gcount > 1) {	/* set up the no gadget via cloning */
					glist->NextGadget = glist+1;
					memcpy((char *)(glist+1),(char *)glist,sizeof(struct Gadget));
					(glist+1)->NextGadget = NULL;
					(glist+1)->LeftEdge = nw->Width - glist->Width - BR_GXOFFSET;
					(glist+1)->GadgetID = BR_NO;
					(glist+1)->GadgetText = (glist->GadgetText)+1;
					memcpy((char *)((glist+1)->GadgetText),(char *)(glist->GadgetText),
					        sizeof(struct IntuiText));
					(glist+1)->GadgetText->LeftEdge = 
					                          (glist->Width - strlen(notext)*8) / 2;
					(glist+1)->GadgetText->IText = notext;
				}
				nw->FirstGadget = glist;
			}
			else { /* can't alloc IText */
				functions->myfree((char *)glist,sizeof(struct Gadget)*gcount);
				if (BR_nw) functions->myfree((char *)BR_nw,sizeof(struct NewWindow));
				return(NULL);
			}
		} /* if glist */
		else { /* can't alloc Gadget(s) */
			if (BR_nw) functions->myfree((char *)BR_nw,sizeof(struct NewWindow));
			return(NULL);
		}
	} /* if gcount */

	/* center the window in the screen horizontally */
/*
	nw->LeftEdge = (functions->screen->Width - nw->Width) >> 1;
	if (nw->LeftEdge < 0) nw->LeftEdge = 0;
	/* put it 1/4 down from the top of the screen */
	nw->TopEdge = (functions->screen->Height - nw->Height) >> 2;
	if (nw->TopEdge < 0) nw->TopEdge=0;
*/
	/* put the window under the pointer */
	nw->LeftEdge = functions->screen->MouseX - (nw->Width >> 1);
	if (nw->LeftEdge < 0) nw->LeftEdge = 0;
	nw->TopEdge = functions->screen->MouseY - (nw->Height >> 2);
	if (nw->TopEdge < 0) nw->TopEdge=0;

	/* now do it */
	if (w = (struct Window *)functions->FlashyOpenWindow(nw)) {
		if (title) flags |= WINDOWDRAG; /* somewhere to put the title */
		functions->EmbossWindowOn(w,flags,title,(short)-1,(short)-1,0,0);
		if (apen < 0) apen = BR_TEXTCOLOR; /* check pen argument values, */
		if (bpen < 0) bpen = BR_FILLCOLOR; /* maybe use defaults */
	
		/* fill window with bpen if not 0 */
		if (bpen) {
			SetDrMd(w->RPort,JAM1);
			BNDRYOFF(w->RPort);
			SetAPen(w->RPort,bpen);
			RectFill(w->RPort, 4, (flags & WINDOWDRAG) ? TITLEBAR_HEIGHT+1 : 2,
			                   w->Width-5, w->Height-3);
		}
		SetAPen(w->RPort,apen);
		SetBPen(w->RPort,bpen);
		SetDrMd(w->RPort,JAM2);
		/* print body text */
		for ( c=p=body,nl=0 ; *c ; nl++ ) {
			for ( ; *c && (*c != '\n') ; c++ ) ; /* find newline or null */
			Move(w->RPort,(center) ? (nw->Width-((c-p)<<3)) >> 1 : BR_LEGROOM,
			     (flags & WINDOWDRAG) ?
			        (nl << 3)+(BR_LINESPACE*nl)+TITLEBAR_HEIGHT+BR_HEADROOM+6 :
			        (nl << 3)+(BR_LINESPACE*nl)+BR_HEADROOM+6 );
			Text(w->RPort,p,c-p); /* print from previous to current position */
			if (*c) p = ++c;      /* if more, point at next printing char */
		}
		if (gcount) {
			functions->EmbossOn(w,BR_YES,1);
			if (gcount > 1) functions->EmbossOn(w,BR_NO,1);
		}
	}
	else { /* window is stuck; free everything */
		functions->myfree((char *)glist->GadgetText,sizeof(struct IntuiText)*gcount);
		functions->myfree((char *)glist,sizeof(struct Gadget)*gcount);
		if (BR_nw) functions->myfree((char *)BR_nw,sizeof(struct NewWindow));
		BR_nw = NULL;
	}
	return(w);
}

/*==========================================================================
* CloseBozoRequest
*
*  INPUT: pointer to Window
* RETURN: none
*
* EFFECT: closes and deallocates
*=========================================================================*/

void CloseBozoRequest(w)

struct Window *w;

{
	struct Gadget *gyes; /* the yes and no gadgets */
	struct Gadget *gno;

	if (gyes = (struct Gadget *)functions->GetGadget(w,BR_YES)) {
		functions->EmbossOff(w,BR_YES);
		functions->myfree((char *)gyes->GadgetText,sizeof(struct IntuiText));
		functions->myfree((char *)gyes,sizeof(struct Gadget));
	}
	if (gno = (struct Gadget *)functions->GetGadget(w,BR_NO)) {
		functions->EmbossOff(w,BR_NO);
		functions->myfree((char *)gno->GadgetText,sizeof(struct IntuiText));
		functions->myfree((char *)gno,sizeof(struct Gadget));
	}
	functions->EmbossWindowOff(w);
	functions->FlashyCloseWindow(w);
	if (BR_nw) functions->myfree((char *)BR_nw,sizeof(struct NewWindow));
	BR_nw = NULL;
}

/*==========================================================================
* BozoRequest
*
*  INPUT: pointer to NewWindow, pointers to title and body text, window flags,
*         apen and bpen colors, center flag, pointer to yes and no gadget text
*
* RETURN: BR_YES, _NO, or _CLOSEWINDOW if everything works, or -1 if error
*
* EFFECT: fake sort of requester type thing for bozos.  Pass it a righteously
*         initialized NewWindow, or NULL for the nw argument and it will use
*         its own NewWindow.  (This nw is allocated and freed within the
*         function; it may be a little slower, but easier.)  title is a
*		 	 pointer to a string to use as a title for the window.  If title 
*			 NULL, the window will not have a drag gadget/title bar.  body is a 
*			 pointer to a string of text to print inside the window.  It can 
*			 contain newline characters ('\n') for multiple lines of text in the 
*			 window.  The height and width of the window is adjusted to accomodate
*			 the text.  flags is the flag bits passed to EmbossedWindowOn().  It 
*			 can be NULL.  If title is not 
*			 NULL, you will automatically get a drag gadget.  If you want any
*			 other flags, you must pass them.  apen is the color for the text.
*			 bpen is the background color for the text, which is always rendered
*			 in JAM2 mode.  Pass -1 for apen or bpen to use the colors (1 and 2
*			 respectively). Set center to center the body text in the window.
*
*			 Do not pass a title or body string that is longer than the width of 
*			 the screen on which the window will open.  You will not like the
*			 result.  The RastPort clips the text, but not before trashing the
*			 window border.
*
*			 This function assumes 8-pixel text.
*
* DEFAULTS: You get these default values if nw == NULL, flags=0, apen and
*           bpen == -1 :
*			 	apen == 0; bpen == 2; flags == BORDERLESS (if title != NULL, 
*         	flags == BORDERLESS | WINDOWDRAG);
*
*==========================================================================*/

long BozoRequest(nw, title, body, flags, apen, bpen, center, yestext, notext)

struct NewWindow *nw;
char const *title;
char const *body;
ULONG flags;
short apen;
short bpen;
int center;
char * const yestext;
char * const notext;

{
	struct Window *w;
	struct IntuiMessage *message;
	long code;
	long class;
	struct Gadget *g;

#pragma msg 104 ignore push
	w = OpenBozoRequest(nw,title,body,flags,apen,bpen,center,yestext,notext);
#pragma msg 104 ignore pop

	if (w) { /* hang out until a gadget gets hit */
		for ( ; ; ) {
			message = (struct IntuiMessage *)functions->GetIntuiMessage(w);
			class = message->Class;
			code = message->Code;
			g = (struct Gadget *)message->IAddress;
			ReplyMsg((struct Message *)message);
			class = functions->SystemGadgets(w,class,g,code);

			if (class == CLOSEWINDOW) {
				code = BR_CLOSEWINDOW;
				break;
			}
			else if (class == GADGETUP) {
				code = g->GadgetID;
				break;
			}
		}
		
		CloseBozoRequest(w);
		return(code);
	}
	else return(BR_ERRORRETURN);
}

