/***
* 		bpl_edwintit.c -- module for bplib.lib to build edit window title
*  	 1993-96 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
  18 Oct 93 | created from prototype in gatesnap.c
  15 Nov 93 | added inpad flag: tool in toolpad has a fake tool->track; after
            | toolizing, the toolized track's name got use instead of "ToolPad"
  14 Nov 96 | you weren't checking all 16 locations in the ToolPad, poophead.
            | and amiga.lib sprintf() won't allow * in a format string
   2 Sep 97 | added "Macro" under the assumption that no track and not in the
	         | toolpad means we're in a macrotool that's being created.
*/


#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
//#include <stdio.h>
#include <clib/alib_stdio_protos.h>

#include "bars.h"
#include "barsmacros.h"
//#include "proto/bpl_inpad_protos.h"
#include "proto/bpl_edwintit_protos.h"

UBYTE InToolPad(struct Tool *);
extern struct Functions * far functions;

/* these are used to construct the string */
#define OPEN_CHAR  ''
#define CLOSE_CHAR ''
/* the minimum string is OPEN_CHAR CLOSE_CHAR if no name or trackname */
#define MIN_CHARS   3

/*==========================================================================
* MakeEditWindowTitle
*
*  INPUT: pointer to Tool, pointer to tool name, maximum number of characters
*         for the window title (not including '\0')
*
* RETURN: char * to the constructed string
*
* EFFECT: allocates and builds a string "name trackname[.n[n]]" and returns the
*         pointer to it.  If name is NULL, then tool->toolid is used.  If the
*         tool is in a pipeline, as much of track->clip.name as allowed by maxchar
*         is copied into the title string.  If the tool is in the ToolPad, the
*         string "ToolPad" is used instead of the track name.  If there are 
*         multiple instances of tool in a track's toollist or in the ToolPad,
*         the string also contains ".n[n]" to indicate which instance of the tool
*         is being edited.  This numeric id is included even if more of the tool 
*         name or track name must be truncated to fit it into a resulting string
*         of not more that maxchar bytes.
*
* CAVEAT: if name==NULL and tool->toolid contains a 0, you will lose as many
*         bytes as follow the 0 in the toolid.  This shouldn't be a problem with
*         a righteous toolid, but is a potential consideration.
*=========================================================================*/

char *MakeEditWindowTitle(tool, name, maxchar)

struct Tool *tool;
const char *name;
long maxchar;

{
	struct Tool *t;     /* scan track->toollist to look for multiple instances */
	char *s;            /* the name string */
	register char *c;   /* pointer into the string when building */
	ULONG length;       /* bytes in the title string */
	UBYTE count;        /* count of instances of this tool in pipe or pad */
	UBYTE thisone = 0;  /* which instance is this tool */
	UBYTE inpad;        /* found in the toolpad? */

	if (maxchar < MIN_CHARS) return(NULL); /* asswipes need not apply */

	/* look first in the toolpad; tool->track has a fake track! */
	count = InToolPad(tool);
	if (count) {
		inpad = 1;
		thisone = count;
	}
	else inpad = 0;
	if (!inpad && tool->track) { /* surely in a pipeline */
		count=0;
		for (t = tool->track->toollist ; t ; t = t->next ) { /* count instances */
			if (t->toolid == tool->toolid) {
				count++;
				if (t == tool) {
					thisone = count; /* we are this instance */
					if (count > 1) break; /* more than one for sure; quit looking */
				}
			}
		}
	}

	/* reckon the length required for the title string */
	if (name) length = strlen(name);
	else length = 4; /* we will use tool->toolid (hope there is no null in it) */
	if (!inpad) {
		if (tool->track) {
			length += strlen(functions->stringtext(tool->track->clip.name));
		}
		else { // macro creation
			length += 5; /* strlen("Macro") */
		}
	}
	else length += 7; /* strlen("ToolPad") */
	if (count > 1) {  /* multiple instances; add room for id ".n[n]"*/
		count = (thisone < 10) ? 1 : 2; /* (this got tested repeatedly) */
		length += count+1;
	}
	else count=0;
	if ( maxchar && (length+MIN_CHARS > maxchar) ) length = maxchar-MIN_CHARS;
	if ( s = functions->myalloc(length+MIN_CHARS+1,MEMF_CLEAR) ) { /* 1 for '\0' */
		if (count) { /* save room for id at expense of tool and track names */
			length -= (count + 1);
		}
		/* build the title string */
#pragma msg 104 ignore push
		if (name) length -= stccpy(s,name,length)-1;
#pragma msg 104 ignore pop
		else {
			memcpy(s,&tool->toolid,4); /* use the toolid instead */
			length -= 4;
		}
		if (length) {
			c = s + strlen(s);
			*c = ' ';
			*(++c) = OPEN_CHAR;
			if (!inpad) {
				if (tool->track) {
					strncpy(++c,functions->stringtext(tool->track->clip.name),length);
				}
				else {
					strcpy(++c,"Macro");
				}
			}
			else {
				strcpy(++c,"ToolPad");
			}
		}
		c = s + strlen(s);
		if (count) {
//			c += sprintf(c,".%*d",count,thisone);
			sprintf(c,".%ld",thisone);  // amiga.lib sprintf won't do substitution
			c += count+1;               // and doesn't return the string length
		}
		*c = CLOSE_CHAR;
	}
	return(s);
}
