/***
* 	bplib_newwindow.c -- module for bplib.lib
*   1993 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
  18 Oct 93 | created from bpl_window.c to support dynamic window titles
*/

#include <exec/types.h>
#include <intuition/intuition.h>
#include <proto/intuition.h>
#include <string.h>
#include "bars.h"
#include "barsmacros.h"
#include "proto/bpl_newwindow_protos.h"

extern struct IntuitionBase *IntuitionBase;
extern struct Functions * far functions;

extern char *MakeEditWindowTitle(struct Tool *, const char *, long );

/*==========================================================================
* NewOpenEmbossedWindow
*
*  INPUT: pointer to NewWindow and tool, window flags, pointer to window title
* RETURN: pointer to NewWindow and pointer to Window in tool->window if
*         opened, else NULL
*
* EFFECT: twiddles with NewWindow and opens embossed window.  if flags==0,
*         defaults are used 
*=========================================================================*/

struct NewWindow *NewOpenEmbossedWindow(new, tool, flags, title)

struct NewWindow *new;
struct Tool *tool;
ULONG flags;
const char *title;

{
	struct NewWindow *nw;
	char *wtitle;

	nw = (struct NewWindow *)functions->DupeNewWindow(new);
	if (nw) {
		if (tool->touched & TOUCH_EDIT) {
			nw->LeftEdge = tool->left;
			nw->TopEdge = tool->top;
			nw->Width = tool->width;
			nw->Height = tool->height;
		}
		nw->Screen = functions->screen;
		nw->Title = 0;
		nw->Flags |= BORDERLESS;
		nw->Flags &= ~0xF;
		nw->DetailPen = nw->BlockPen = 0;
		tool->window = (struct Window *)functions->FlashyOpenWindow(nw);
		if (tool->window) {
			/* subtracting 55 allows for close & front gadget; drag text offset */
			wtitle = MakeEditWindowTitle(tool,title,(nw->Width-55)>>3); /* 8 pixel font alert */
			functions->EmbossWindowOn(tool->window, flags ?
			                          flags : WINDOWCLOSE|WINDOWDEPTH|WINDOWDRAG,
											  wtitle,(short)-1,(short)-1,0,0);
		}
		else {
			functions->DeleteNewWindow(nw);
			nw=NULL;
		}
	}
	return(nw);
}

/*==========================================================================
* NewCloseEmbossedWindow
*
*  INPUT: pointer to tool and NewWindow created with DupeNewWindow
* RETURN: none
*
* EFFECT: saves window stuff in tool, closes tool->window, deletes new window
*=========================================================================*/

void NewCloseEmbossedWindow(tool, nw)

struct Tool *tool;
struct NewWindow *nw;

{
	struct Gadget *g = functions->GetGadget(tool->window,EMBOSSED_DRAG);

	tool->left = tool->window->LeftEdge;
	tool->top = tool->window->TopEdge;
	tool->width = tool->window->Width;
	tool->height = tool->window->Height;
	tool->touched |= TOUCH_EDIT;
	if (tool->window->MenuStrip) ClearMenuStrip(tool->window);
	if (g) functions->myfree(g->GadgetText->IText,strlen(g->GadgetText->IText)+1);
	functions->EmbossWindowOff(tool->window);
	functions->FlashyCloseWindow(tool->window);
	tool->window = NULL;
	if (nw) functions->DeleteNewWindow(nw);
}
