/***
* 	bplib_snapshot.c -- module for bplib.lib
*   1993,1994 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
  18 Sep 93 | created
  12 Nov 93 | added PostSnapshot()
  17 Jan 94 | modified PostSnapshot() to get rid of extraneous existing sysex;
            | modified BuildSnapshot() to use edit marker time as default time.
  12 Feb 94 | PostSnapshot() refreshes track window now (B&PP v2.1+)
  20 Nov 96 | test track left edit flag marker for less than 0
*/

#include <exec/types.h>
#include <exec/memory.h>
#include <proto/exec.h>
#include <intuition/intuition.h>
#include <proto/intuition.h>
#include <stddef.h>
#include <string.h>
#include "bars.h"
#include "edit.h"
#include "bplib.h"
#include "proto/bpl_snapshot_protos.h"

extern struct Functions * far functions;

/* the minimum number of bytes required (SOX 0x01 toolID(4) EOX NULL) */
#define MIN_BYTES 8

/*==========================================================================
* BuildSnapshot
*
*  INPUT: pointer to tool, time, pointer to sysex data, data array count
* RETURN: pointer to StringEvent, or NULL
*
* EFFECT: allocates a StringEvent and constructs a sysex in StringEvent.string.
*         The StringEvent.time is the argument passed unless time=-1 in which
*         case the time is set to the edit->begin marker if the track window
*         is open, or the functions->cutin marker if the track window is not
*         open.
*
*			 !The UBYTE array has to have printable ASCII in it or allocstring()
*         doesn't.
*=========================================================================*/

struct StringEvent *BuildSnapshot(tool, time, data, count)

struct Tool *tool;
long time;
UBYTE *data;
ULONG count;

{
	struct StringEvent *event = (struct StringEvent *)functions->allocevent();
	ULONG length = count + MIN_BYTES;
	char *snap; /* fake string */

	if (event) {
		if (snap = (UBYTE *)functions->myalloc(length,MEMF_CLEAR)) {
			memset(snap,'m',length); /* need characters to trick allocstring */
			snap[length-1] = 0;
			if (event->string = functions->allocstring(snap)) {
				/* set up the StringEvent */
				event->next = NULL;
				/* divine a time for the event */
				if (time > -1) event->time = time; /* do it at the caller's time */
				else if (tool->track->edit && tool->track->edit->inedit) { /* use begin edit marker */
					event->time = (tool->track->edit->begin < 0) ? 0 : tool->track->edit->begin;
				}
				else { /* use main window edit marker */
					event->time = (functions->cutin < 0) ? 0 : functions->cutin;
				}
				event->type = EVENT_SYSX;
				event->status = MIDI_SYSX;
				event->length = event->string->length;
				event->tool = tool->next;
				/* set up the sysex String */
				event->string->string[0] = (char)MIDI_SYSX;
				event->string->string[1] = TOOL_SYSEX_ID;
				memcpy((void *)(&event->string->string[2]),(void *)(&tool->toolid),4);
				memcpy((void *)(&event->string->string[6]),(void *)data,count);
				event->string->string[length-2] = (char)MIDI_EOX;
			}
			else { /* couldn't allocstring() */
				functions->freeevent((struct Event *)event);
				event = NULL;
			}
			functions->myfree(snap,length);
		}
		else { /* couldn't alloc string for sysex data */
			functions->freeevent((struct Event *)event);
			event = NULL;
		}
	}
	return(event);
}


extern char *checksysex(struct StringEvent *, BYTE, ULONG);

/*==========================================================================
* PostSnapshot
*
*  INPUT: pointers to Tool and StringEvent
* RETURN: none
*
* EFFECT: figures out which Clip to put the event in and does it.  If there
*         is another sysex event for the same tool within edit->insertlength-1
*         of the new event, the existing event is freed (i.e., the closest
*         sysex event can occur edit->insertlength from this one).
*=========================================================================*/

void PostSnapshot(tool,event)

struct Tool *tool;
struct StringEvent *event;

{
	register struct Event *e;
	struct Clip *clip;
	struct Event *x; /* event to free */
	short length;

	/* track window open? use edit struct */
	if (tool->track->edit && tool->track->edit->inedit) {
		clip = &tool->track->edit->clip;
		length = tool->track->edit->insertlength;
	}
	else { /* or use track clip and master default note */
		clip = &tool->track->clip;
		if (functions->masteredit) length = functions->masteredit->insertlength;
		else length = 192;
	}
	/* find first event with time in range for consideration */
	for (e=clip->events.first ; 
	     e && (e->time <= event->time-length) ; e=e->next )
		;

	/* now, get rid of this tool's sysex at event->time +/- default note length */
	while ( e && (e->time < event->time+length) ) {
		if (e->type & EVENT_SYSX) {
			if (checksysex((struct StringEvent *)e,TOOL_SYSEX_ID,tool->toolid)) {
				x = e;
				e = e->next;
				clip->events.first = (struct Event *) functions->List_Remove (
				        (struct ListItem *)clip->events.first,(struct ListItem *)x);
				functions->freeevent(x);
			}
		}
		else e = e->next;
	}

	/* the list has been cleaned out; stick the new one into it */
	event->next = (struct StringEvent *)clip->events.first;
	clip->events.first = functions->sorteventlist((struct Event *)event);
	if (tool->track->edit->window) /* refresh to show the event */
		functions->updateobject(&functions->inittempo,0);
}

