/***
* 		ConfigLib_Window.c -- module for ConfigLib.lib
*  	 1993 Another Option   All rights reserved.
***/

/*
============================================================================
|    DATE   | WHAT
============================================================================
   3 Feb 93 | created
*/

#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>

#include "bars.h"
#include "configlib.h"

extern struct Functions * far functions;

/*==========================================================================
* OpenConfigWindow
*
*  INPUT: pointers to NewWindow, ControlConfig; window flags; window title
* RETURN: pointer to duplicated NewWindow
*
* EFFECT: duplicates NewWindow, opens an embossed window stashing the Window
*         pointer in the ControlConfig structure.  FUNKY NOTE: to ensure
*         the tool's going to display the proper control configuration,
*         ControlConfig.controller is forced to point at the first element
*         of the tool's ControllerInfo array.
*=========================================================================*/

struct NewWindow *OpenConfigWindow(long tool, struct NewWindow *confignw,
                                   ULONG flags, char *title)
{
	struct NewWindow *nw;
   struct ControlConfig *config = &(((struct CC_Tool *)tool)->_config);

   /* make sure to point at this tool's ControllerInfo arary */
   config->controller = &((struct CC_Tool *)tool)->_cinfo[0];
	nw = (struct NewWindow *)(*functions->DupeNewWindow)(confignw);
	if (nw) {
		if (config->touched & TOUCH_EDIT) {
			nw->TopEdge = config->top;
			nw->LeftEdge = config->left;
		}
		nw->Screen = functions->screen;
		nw->Title = 0;
		nw->Flags |= BORDERLESS;
		nw->Flags &= ~0xF;
		nw->DetailPen = nw->BlockPen = 0;
		config->window = (struct Window *)(*functions->FlashyOpenWindow)(nw);
		if (config->window) {
			(*functions->EmbossWindowOn)(config->window,
                              flags?flags:WINDOWCLOSE|WINDOWDEPTH|WINDOWDRAG,
		       						title,(short)-1,(short)-1,0,0);
		}
		else {
			(*functions->DeleteNewWindow)(nw);
			nw = NULL;
		}
	}
	return(nw);
}

/*==========================================================================
* CloseConfigWindow
*
*  INPUT: pointer to tool (cast to long) and pointer to duped NewWindow
* RETURN: none
*
* EFFECT: stashes window position in ControlConfig, closes the window, and
*         frees the NewWindow
*=========================================================================*/

void CloseConfigWindow(long tool, struct NewWindow *nw)

{
   struct ControlConfig *config = &(((struct CC_Tool *)tool)->_config);

	config->left=config->window->LeftEdge;
	config->top = config->window->TopEdge;
	config->touched |= TOUCH_EDIT;
	(*functions->EmbossWindowOff)(config->window);
	(*functions->FlashyCloseWindow)(config->window);
	config->window = NULL;
	(*functions->DeleteNewWindow)(nw);
}

